using System;
using System.IO;
using System.Drawing;

namespace GifTran
{
	/// <summary>
	/// Author : Kerry Wong
	/// 5/2004
	/// http://www.kerrywong.com/
	/// </summary>
	public class TransparentGIF
	{
		private string fn;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="fn">the full path of the gif file to be made transparent</param>
		public TransparentGIF(string fn)
		{
			this.fn = fn;
		}

		/// <summary>
		/// Make the image transparent. 
		/// The input is the color which you want to make transparent.
		/// </summary>
		/// <param name="R"></param>
		/// <param name="G"></param>
		/// <param name="B"></param>
		/// <returns></returns>
		public bool MakeTransparent(byte R, byte G, byte B) {
			FileStream fin = File.OpenRead(fn);
			string tempFileName = @"c:\__$$temp$gif$$__.temp";
			FileStream fout = File.Create(tempFileName);
			int count = 0;
			byte[] buf = new byte[256];
			byte transparentIdx = 0;

			//header
			count = fin.Read(buf,0,13);
			if ((buf[0]!=71) || (buf[1]!=73) || (buf[2]!=70)) return false; //GIF

			fout.Write(buf,0,13);
			
			int i = 0;
			if ((buf[10] & 0x80) > 0) {
				i = 1<<((buf[10] & 7)+1) == 256 ? 256:0;
			}

			for (;i!=0; i--) {
				fin.Read(buf,0,3);
				if ((buf[0]==R) && (buf[1]==G) && (buf[2]==B)) {
					transparentIdx = (byte) (256-i);
				}
				fout.Write(buf,0,3);
			}	

			bool gcePresent = false;
			while (true){
				fin.Read(buf,0,1);
				fout.Write(buf,0,1);
				if (buf[0]!=0x21) break;
				fin.Read(buf,0,1);
				fout.Write(buf,0,1);
				gcePresent = (buf[0] == 0xf9);
				while (true) {
					fin.Read(buf,0,1);
					fout.Write(buf,0,1);
					if (buf[0] == 0) break;
					count = buf[0];
					if (fin.Read(buf,0,count) != count) return false;
					if (gcePresent) {
						if (count ==4) {
							buf[0] |= 0x01;
							buf[3] = transparentIdx;
						}
					}
					fout.Write(buf,0,count);
				}
			}
			while (count>0) {
				count = fin.Read(buf,0,1);
				fout.Write(buf,0,1);
			}	
			fin.Close();
			fout.Flush();
			fout.Close();
			File.Delete(fn);
			File.Move(tempFileName,fn);
			File.Delete(tempFileName);
			return true;
		}
	}
}
